// ----------------------------------------------------------------------------
//
// Matrix multiplication (thread pool version with futures).
// (c) Wolfgang Schreiner, 2017.
//
// ----------------------------------------------------------------------------

import java.util.*;
import java.util.concurrent.*;

public class MatMultFuture
{
  // matrix dimension
  private static int N; 

  // number of threads
  private static int T;
 
  // shared data used by threads
  private static double[][] A;
  private static double[][] B;
  private static double[][] C;

  // -------------------------------------------------------------------------
  // 
  // a thread in charge of filling C[i]
  //
  // -------------------------------------------------------------------------
  private static final class MultResult implements Callable<double[]>
  {
    private int i;
    public MultResult(int i)
    {
      this.i = i;
    }
    public double[] call() throws Exception
    {
      double[] C = new double[N];
      for (int j = 0; j < N; j++) 
      {
	    C[j] = 0;
	    for (int k = 0; k < N; k++) 
	      C[j] += A[i][k]*B[k][j];
      }
      return C;
    }
  }

  // -------------------------------------------------------------------------
  // 
  // multiply A by B giving C
  // 
  // -------------------------------------------------------------------------
  private static void multiply() 
  {
    ExecutorService pool = Executors.newFixedThreadPool(T);
    Vector<Future<double[]>> result = new Vector<Future<double[]>>(N);
    for (int i = 0; i < N; i++) 
      result.add(pool.submit(new MultResult(i))); 
    pool.shutdown();
    
    try
    {
      for (int i = 0; i < N; i++)
        C[i] = result.get(i).get();
    }
    catch(InterruptedException e) { }
    catch(ExecutionException e) { }
  }

  // ----------------------------------------------------------------------------
  //
  // main program
  //
  // ----------------------------------------------------------------------------
  public static void main(String[] args) 
  {
    if (args.length != 2)
    {
      System.out.println("Usage: MatMultThreads <N> <T>");
      return;
    }
    try
    {
      N = Integer.parseInt(args[0]);
      T = Integer.parseInt(args[1]);
    }
    catch(NumberFormatException e)
    {
      System.out.println("Invalid argument (number expected): " + args[0]);
      return;
    } 

    A = new double[N][N];
    B = new double[N][N];
    C = new double[N][];
    multiply();
  }
}

