// ----------------------------------------------------------------------------
//
// Matrix multiplication (thread pool version with recursion).
// (c) Wolfgang Schreiner, 2017.
//
// ----------------------------------------------------------------------------

import java.util.*;
import java.util.concurrent.*;

public class MatMultRec
{
  // matrix dimension
  private static int N; 

  // number of threads
  private static int T;
 
  // shared data used by threads
  private static double[][] A;
  private static double[][] B;
  private static double[][] C;

  // -------------------------------------------------------------------------
  // 
  // a thread in charge of filling C[i]
  //
  // -------------------------------------------------------------------------
  private static final class MultRec extends RecursiveAction
  {
    private int begin; 
    private int end;
    public MultRec(int begin, int end)
    {
      this.begin = begin;
      this.end = end;
    }
    public void compute() 
    {
      if (begin == end-1)
      { 
      	int i = begin;
        for (int j = 0; j < N; j++) 
        {
	      C[i][j] = 0;
	      for (int k = 0; k < N; k++) 
	        C[i][j] += A[i][k]*B[k][j];
	    }
      }
      else if (begin < end)
      {
      	int mid = (begin+end)/2;
        invokeAll(new MultRec(begin, mid), new MultRec(mid, end));
      }
    }
  }

  // -------------------------------------------------------------------------
  // 
  // multiply A by B giving C
  // 
  // -------------------------------------------------------------------------
  private static void multiply() 
  {
    ForkJoinPool pool = new ForkJoinPool(T);
    ForkJoinTask<Void> task = pool.submit(new MultRec(0,N)); 
    task.join();
  }

  // ----------------------------------------------------------------------------
  //
  // main program
  //
  // ----------------------------------------------------------------------------
  public static void main(String[] args) 
  {
    if (args.length != 2)
    {
      System.out.println("Usage: MatMultThreads <N> <T>");
      return;
    }
    try
    {
      N = Integer.parseInt(args[0]);
      T = Integer.parseInt(args[1]);
    }
    catch(NumberFormatException e)
    {
      System.out.println("Invalid argument (number expected): " + args[0]);
      return;
    } 

    A = new double[N][N];
    B = new double[N][N];
    C = new double[N][N];
    multiply();
  }
}

