// ----------------------------------------------------------------------------
//
// Matrix multiplication (multi-threaded version).
// (c) Wolfgang Schreiner, 2017.
//
// ----------------------------------------------------------------------------
public class MatMultThreads
{
  // matrix dimension
  private static int N; 

  // number of threads
  private static int T;
 
  // shared data used by threads
  private static double[][] A;
  private static double[][] B;
  private static double[][] C;

  // -------------------------------------------------------------------------
  // 
  // a thread in charge of filling C[begin..end-1]
  //
  // -------------------------------------------------------------------------
  private static final class MultThread extends Thread
  {
    private int begin;
    private int end;
    public MultThread(int begin, int end)
    {
      this.begin = begin;
      this.end = end;
    }
    public void run()
    {
      for (int i = begin; i < end; i++) 
      {
        for (int j = 0; j < N; j++) 
        {
	      C[i][j] = 0;
	      for (int k = 0; k < N; k++) 
	        C[i][j] += A[i][k]*B[k][j];
        }
      }
    }
  }

  // -------------------------------------------------------------------------
  // 
  // multiply A by B giving C
  // 
  // -------------------------------------------------------------------------
  private static void multiply() 
  {
    int n = N/T;
    
    Thread[] thread = new MultThread[T];
    for (int i = 0; i < T; i++) 
    {
	  thread[i] = new MultThread(i*n, Math.min((i+1)*n,N));
      thread[i].start();
    }

    try
    {
      for (int i = 0; i < T; i++) 
        thread[i].join();
    }
    catch(InterruptedException e) { }
  }

  // ----------------------------------------------------------------------------
  //
  // main program
  //
  // ----------------------------------------------------------------------------
  public static void main(String[] args) 
  {
    if (args.length != 2)
    {
      System.out.println("Usage: MatMultThreads <N> <T>");
      return;
    }
    try
    {
      N = Integer.parseInt(args[0]);
      T = Integer.parseInt(args[1]);
    }
    catch(NumberFormatException e)
    {
      System.out.println("Invalid argument (number expected): " + args[0]);
      return;
    } 

    A = new double[N][N];
    B = new double[N][N];
    C = new double[N][N];
    multiply();
  }
}

