// ----------------------------------------------------------------------------
//
// Matrix multiplication (worker version).
// (c) Wolfgang Schreiner, 2017.
//
// ----------------------------------------------------------------------------
public class MatMultWorkers
{
  // matrix dimension
  private static int N; 

  // number of threads
  private static int T;
 
  // shared data used by threads
  private static double[][] A;
  private static double[][] B;
  private static double[][] C;

  // number of result rows already computed
  private static int rows;

  // --------------------------------------------------------------------------
  // 
  // a thread computing dynamically selected rows of C
  // 
  // --------------------------------------------------------------------------
  private static final class MultWorker extends Thread
  {
    public void run()
    {
      while (true)
      {
        int i;
        synchronized (C)
        {
          i = rows;
          rows++;
        }
        if (i >= N) return;
        for (int j = 0; j < N; j++) 
        {
	      C[i][j] = 0;
	      for (int k = 0; k < N; k++) 
	        C[i][j] += A[i][k]*B[k][j];
        }
      }
    }
  }

  // --------------------------------------------------------------------------
  // 
  // multiply A by B giving C
  // 
  // --------------------------------------------------------------------------
  private static void multiply() 
  {
    int n = N/T;
    
    Thread[] thread = new MultWorker[T];
    for (int i = 0; i < T; i++) 
    {
	  thread[i] = new MultWorker();
      thread[i].start();
    }

    try
    {
      for (int i = 0; i < T; i++) 
        thread[i].join();
    }
    catch(InterruptedException e) { }
  }

  // ----------------------------------------------------------------------------
  //
  // main program
  //
  // ----------------------------------------------------------------------------
  public static void main(String[] args) 
  {
    if (args.length != 2)
    {
      System.out.println("Usage: MatMultWorkers <N> <T>");
      return;
    }
    try
    {
      N = Integer.parseInt(args[0]);
      T = Integer.parseInt(args[1]);
    }
    catch(NumberFormatException e)
    {
      System.out.println("Invalid argument (number expected): " + args[0]);
      return;
    } 

    A = new double[N][N];
    B = new double[N][N];
    C = new double[N][N];
    rows = 0;
    multiply();
  }
}

